package codex

import (
  "fmt"
  "math"
  "strconv"
)

type CodexLVar struct {
  id            string
  PacketLength  int
  maxLength     int
  lfmt          string
  length        int
}

func (c *CodexLVar) init(cfg map[string]interface{}) {
  if v, ok := cfg["id"].(string); ok {
    c.id = v
  } else {
    panic(fmt.Sprintf("Invalid CodexLVar.ID %+v", cfg))
  }
  if v, ok := cfg["length"].(int); ok {
    c.length = v
  } else if cfg["length"] == nil {
    c.length = 0
  } else {
    panic(fmt.Sprintf("Invalid CodexLVar.Length %+v", cfg))
  }
  c.maxLength = int(math.Pow(10, float64(c.PacketLength))) - 1
  c.lfmt = fmt.Sprintf("%%0%dd", c.PacketLength)
}

func (c *CodexLVar) ID() string {
  return c.id
}

func (c *CodexLVar) decode(data []byte, pos int) (string, int, error) {
  ppos := pos + c.PacketLength
  if len(data) < ppos {
    return "", pos, fmt.Errorf("incomplete packet field %s %d < %d", c.id, len(data), ppos)
  }
  plen, err := strconv.Atoi(string(data[pos:ppos]))
  if err != nil {
    return "", pos, fmt.Errorf("Invalid lvar %s length [%s] %s", c.id, string(data[pos:ppos]), err)
  }
  ppos1 := ppos
  ppos = ppos + plen
  if len(data) < ppos {
    return "", pos, fmt.Errorf("incomplete packet field %s %d < %d", c.id, len(data), ppos)
  }
  return string(data[ppos1:ppos]), ppos, nil
}

func (c *CodexLVar) Decode(data []byte, pos int, value interface{}) (int, error) {
  var err error
  if vm, ok := value.(map[string]interface{}); ok {
    vm[c.id], pos, err = c.decode(data, pos)
    if err != nil {
      return pos, err
    }
    return pos, nil
  }
  panic(fmt.Sprintf("Invalid value type %+v", value))
}

func (c *CodexLVar) Encode(data []byte, pos int, value interface{}) (int, error) {
  if vm, ok := value.(map[string]interface{}); ok {
    if v, ok := vm[c.id].(string); ok {
      ppos := pos + c.PacketLength
      lv := len(v)
      if lv > c.maxLength {
        sl := fmt.Sprintf(c.lfmt, c.maxLength)
        copy(data[pos:ppos], []byte(sl))
        copy(data[ppos:ppos+c.maxLength], []byte(v[0:c.maxLength]))
        return ppos + c.maxLength, nil
      }
      sl := fmt.Sprintf(c.lfmt, lv)
      copy(data[pos:ppos], []byte(sl))
      copy(data[ppos:ppos+lv], []byte(v))
      return ppos + lv, nil
    }
    panic(fmt.Sprintf("Invalid value type [%s] %+v", c.id, vm))
  }
  panic(fmt.Sprintf("Invalid value type [%s] %+v", c.id, value))
}
